;;;; coerce-test.scm

(use testbase testbase-output-compact)
(use coerce)

(define-test coerce-test "Coerce"

  (expect-equal "lvs"
    '((#\s #\t #\r #\i #\n #\g) #(#\s #\t #\r #\i #\n #\g) "string")
    (coerce-all '("string" (#\s #\t #\r #\i #\n #\g) #(#\s #\t #\r #\i #\n #\g))
                '(list vector string)))

  (expect-success "extn"
    (extend-coerce 'undefined (lambda (x) (eq? (void) x))
      (case-coerce
        [(char)     #\nul]
        [(number)   +nan]
        [(string)   ""]
        [(symbol)   ||]
        [(list)     (list)]
        [(vector)   (vector)])))

  (expect-eq "ut" 'undefined (type-of (void)))
  (expect-equal "ns" "" (coerce (void) 'string))
  
  (expect-success "string extension"
    (extend-coerce 'string string?
      (case-coerce
        [(undefined) (if (string=? "" object) (void) (on-error))])))

  (expect-failure "su1" (coerce "abc" 'undefined))
  (expect-eq "su2" (void) (coerce "" 'undefined))

  (let-values ([(pred proc) (coerce-extension 'string)]) 
    (expect-success "override string extension"
      (extend-coerce 'string string?
        (case-coerce
          [(input-port)  `(open-input-file ,object)]
          [(output-port) `(open-output-file ,object)]
          [else (proc object 'string on-error)]))) )

  (expect-equal "sp" '(open-output-file "foo") (coerce "foo" 'output-port))
)

(test::for-each (cut test::styler-set! <> test::output-style-compact))
(run-test "Coerce Tests")
